/*---------------------------------------------------------------------------
	
	Creator: TheCodingBeast - TheCodingBeast.com
	This work is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International License. 
	To view a copy of this license, visit http://creativecommons.org/licenses/by-nc-sa/4.0/
	
---------------------------------------------------------------------------*/

--[[---------------------------------------------------------
	Spawn Dealer
-----------------------------------------------------------]]
util.AddNetworkString("TCBDealerMenu")
util.AddNetworkString("TCBDealerSpawn")
util.AddNetworkString("TCBDealerPurchase")
util.AddNetworkString("TCBDealerSell")
util.AddNetworkString("TCBDealerChat")
util.AddNetworkString("TCBDealerStore")

--[[---------------------------------------------------------
	Database Setup
-----------------------------------------------------------]]
function TCBDealer.databaseSetup()

	--> Compatibility
	local AUTOINCREMENT = MySQLite.isMySQL() and "AUTO_INCREMENT" or "AUTOINCREMENT"

	--> Table
	MySQLite.query([[
		CREATE TABLE IF NOT EXISTS tcb_cardealer (
			id INTEGER NOT NULL PRIMARY KEY ]]..AUTOINCREMENT..[[,
			steamID VARCHAR(50) NOT NULL,
			vehicle VARCHAR(255) NOT NULL
		)
	]])

end
hook.Add("DarkRPDBInitialized", "TCBDealer.databaseSetup", TCBDealer.databaseSetup)

--[[---------------------------------------------------------
	Spawn Dealer
-----------------------------------------------------------]]
function TCBDealer.spawnDealer()

	--> Precache
	if TCBDealer.settings.precache then
		for k,v in pairs(TCBDealer.vehicleTable) do
			
			local vehicleList = list.Get("Vehicles")[k]
			if !vehicleList then return end

			util.PrecacheModel(vehicleList.Model)

		end	
	end

	--> Version
	timer.Create("CarDealerFiVersion", 10, 1, function()
		TCBDealer.versionCheck()
	end)

end
hook.Add("InitPostEntity", "TCBDealer.spawnDealer", TCBDealer.spawnDealer)

--[[---------------------------------------------------------
	Version Check
-----------------------------------------------------------]]
local versionCheck = 0
function TCBDealer.versionCheck()

	--> Variables
	versionCheck = versionCheck+1;
	local newVersion = nil

	--> HTTP
	http.Fetch("https://raw.githubusercontent.com/TheCodingBeast/TCB_CarDealer/master/version.txt",
		function(body, len, headers, code)

			--> Variables
			newVersion = tonumber(body)

			--> Check
			if TCBDealer.version < newVersion then
				timer.Create("CarDealerVersion", 30, 0, function()
					MsgC(Color(0, 255, 0), "[TCB] There is a new version of 'TCB Car Dealer' available.\n")
				end)
			else
				MsgC(Color(0, 255, 0), "[TCB] Car Dealer is up to date.\n")
			end

		end,
		function(error)

			--> Warn
			if versionCheck != 1 then
				MsgC(Color(255, 0, 0), "[TCB] There was an error verifying the version.\n"..error.."\n")
			end

			--> Timer
			timer.Create("CarDealerReVersion", 10, 1, function()
				TCBDealer.versionCheck()
			end)

		end
	)

end


--[[---------------------------------------------------------
	Spawn Vehicle
-----------------------------------------------------------]]
function TCBDealer.spawnVehicle(length, ply)

	--> Network
	local vehID = net.ReadString()
	local dealerID = net.ReadInt(32)
	local hours = net.ReadFloat()
	hours = math.Round( hours )

	if !ply:canAfford( TCBDealer.settings.rentCost * hours ) then
		if ply:canAfford( TCBDealer.settings.rentCost ) then
			DarkRP.notify( ply, 1, 4, "You can't afford to rent a car for that long." )
		else
			DarkRP.notify( ply, 1, 4, "You can't afford to rent a car." )
		end
		return
	end

	if IsValid( ply:GetNWEntity("currentVehicle") ) then
		DarkRP.notify( ply, 1, 4, "You already have a car out!" )
		return
	end

	--> Vehicle
	if !TCBDealer.vehicleTable[vehID] then 
		DarkRP.notify(ply, 1, 4, "The requested vehicle can't be spawned.") 
		return 
	end
	vehicle = TCBDealer.vehicleTable[vehID]

	--> CustomCheck
	if vehicle.customCheck and !vehicle.customCheck(ply) then 
		if vehicle.CustomCheckFailMsg then
			DarkRP.notify(ply, 1, 4, vehicle.CustomCheckFailMsg)
		else
			DarkRP.notify(ply, 1, 4, "This vehicle is currently not available for you.")
		end
		return
	end
		
	--> Function
	function spawnCode()

		--> Current
		TCBDealer.removeVehicle(ply)
		--> Dealer
		local dealer = Entity( dealerID )
		
		local dealerResult = TCBDealer.dealerRange(ply, dealer)
		if !dealerResult then
			DarkRP.notify(ply, 1, 4, "You are not in range of the car dealer!") 
			return 
		end
		--> Spawns
		local spawnPoint = {}
		local debugPoint = ""

		if TCBDealer.settings.checkSpawn then
			for k,v in pairs(dealer.spawns) do
				local entities = ents.FindInBox(Vector(v.pos.x + 100, v.pos.y + 100, v.pos.z - 150), Vector(v.pos.x - 100, v.pos.y - 100, v.pos.z + 150))
				
				local found = 0
				for _,v in pairs(entities) do
					if v:GetClass() != "physgun_beam" then
						found = 1

						if TCBDealer.settings.debug then
							debugPoint = debugPoint..v:GetClass()..", "
						end

						break
					end
				end

				if found == 0 then
					spawnPoint = v
					break
				end
			end
		else
			spawnPoint = dealer:GetPos() + dealer:GetAngles():Forward() * 50
		end

		--> Spawn
		local vehicleList = list.Get("Vehicles")[vehID]
		if !vehicleList then return end

		local spawnedVehicle = ents.Create(vehicleList.Class)
		if !spawnedVehicle then return end

		spawnedVehicle:SetModel(vehicleList.Model)

		if vehicleList.KeyValues then
			for k, v in pairs(vehicleList.KeyValues) do
				spawnedVehicle:SetKeyValue(k, v)
			end
		end

		spawnedVehicle.VehicleTable = vehicleList

		spawnedVehicle:SetPos(spawnPoint)
		spawnedVehicle:SetAngles(dealer:GetAngles())
		spawnedVehicle:Spawn()
		spawnedVehicle:Activate()

		spawnedVehicle:keysOwn(ply)
		spawnedVehicle:keysLock()

		spawnedVehicle:SetNWString("dealerName", vehicle.name or vehicleList.Name)
		spawnedVehicle:SetNWString("dealerClass", vehID)

		gamemode.Call(PlayerSpawnedVehicle, ply, spawnedVehicle)
		ply:SetNWEntity("currentVehicle", spawnedVehicle)

		--> Color
		if vehicle.color then
			spawnedVehicle:SetColor(vehicle.color)
		elseif TCBDealer.settings.colorPicker then

			--> Variables
			local varColR = ply:GetInfoNum("tcb_cardealer_r", 0)
			local varColG = ply:GetInfoNum("tcb_cardealer_g", 0)
			local varColB = ply:GetInfoNum("tcb_cardealer_b", 0)

			--> Checks
			if varColR < 0 then varColR = 0 elseif varColR > 255 then varColR = 255 end
			if varColG < 0 then varColG = 0 elseif varColG > 255 then varColR = 255 end
			if varColB < 0 then varColB = 0 elseif varColB > 255 then varColB = 255 end

			--> Set Color
			spawnedVehicle:SetColor(Color(varColR, varColG, varColB, 255))

		elseif TCBDealer.settings.randomColor then
			spawnedVehicle:SetColor(Color(math.random(0, 255), math.random(0, 255), math.random(0, 255), 255))
		end

		--> Test Drive

		if timer.Exists("vehrent_"..ply:EntIndex()) or timer.Exists("vehrent_warn_"..ply:EntIndex()) then
			timer.Remove("vehrent_"..ply:EntIndex())
			timer.Remove("vehrent_warn_"..ply:EntIndex())
		end

		ply:addMoney( -hours * TCBDealer.settings.rentCost )

		timer.Create("vehrent_"..ply:EntIndex(), ( hours * 60 * 60 ) - 600, 1, function()

			DarkRP.notify( ply, 1, 4, "You have 10 minutes left to drive this vehicle!" )

			timer.Create( "vehrent_warn_" .. ply:EntIndex(), 600, 1, function()
				if IsValid(ply) then
					ply:ExitVehicle()
					if IsValid( ply:GetNWEntity("currentVehicle") ) then
						ply:GetNWEntity("currentVehicle"):Remove()
					end
				end

				net.Start("TCBDealerChat")
					net.WriteString("Your rent time ran out!")
				net.Send(ply)

			end )

		end)

		net.Start("TCBDealerChat")
			net.WriteString("You have rented this vehicle for "..hours.." hours!")
		net.Send(ply)

		ply:EnterVehicle(spawnedVehicle)

	end

	spawnCode()
end
net.Receive("TCBDealerSpawn", TCBDealer.spawnVehicle)

--[[---------------------------------------------------------
	Store Vehicle
-----------------------------------------------------------]]
function TCBDealer.storeVehicle(length, ply)

	--> Network
	local dealerID = net.ReadInt(32)
	local timeLeft = 0
	if timer.Exists( "vehrent_" .. ply:EntIndex() ) then
		timeLeft = math.max( timer.TimeLeft( "vehrent_" .. ply:EntIndex() ), 0 )
	else
		timeLeft = math.max( timer.TimeLeft( "vehrent_warn_" .. ply:EntIndex() ), 0 )
	end
	timeLeft = math.floor( timeLeft / 60 / 60 )

	--> Dealer
	
	local dealer = Entity( dealerID )
	
	local dealerResult = TCBDealer.dealerRange(ply, dealer)
	if !dealerResult then
		DarkRP.notify(ply, 1, 4, "You are not in range of the car dealer!") 
		return 
	end

	--> Vehicle
	local currentVehicle = ply:GetNWEntity("currentVehicle")
	if IsValid(currentVehicle) and currentVehicle:GetPos():Distance(ply:GetPos()) <= TCBDealer.settings.storeDistance then
		TCBDealer.removeVehicle(ply)
		DarkRP.notify(ply, 3, 4, "Your vehicle was returned and you were refunded " .. DarkRP.formatMoney( timeLeft * TCBDealer.settings.rentCost ) .. "!" )
		ply:addMoney( timeLeft * TCBDealer.settings.rentCost )
		return
	else
		DarkRP.notify(ply, 1, 4, "The vehicle is not in range.") 
		return
	end 

end
net.Receive("TCBDealerStore", TCBDealer.storeVehicle)

--[[---------------------------------------------------------
	Remove Vehicle
-----------------------------------------------------------]]
function TCBDealer.removeVehicle(ply)
	local currentVehicle = ply:GetNWEntity("currentVehicle")
	if IsValid(currentVehicle) then
		local timeLeft = 0
		if timer.Exists( "vehrent_" .. ply:EntIndex() ) then
			timeLeft = math.max( timer.TimeLeft( "vehrent_" .. ply:EntIndex() ), 0 )
		else
			timeLeft = math.max( timer.TimeLeft( "vehrent_warn_" .. ply:EntIndex() ), 0 )
		end
		timeLeft = math.floor( timeLeft / 60 / 60 )
		ply:addMoney( timeLeft * TCBDealer.settings.rentCost )
		timer.Remove( "vehrent_" .. ply:EntIndex() )
		timer.Remove( "vehrent_warn_" .. ply:EntIndex() )
		--> Remove
		currentVehicle:Remove()

	end
end
hook.Add("PlayerDisconnected", "TCBDealer.removeVehicle", TCBDealer.removeVehicle)

--[[---------------------------------------------------------
	Player Changed
-----------------------------------------------------------]]
function TCBDealer.playerChanged(ply)
	local currentVehicle = ply:GetNWEntity("currentVehicle")
	if IsValid(currentVehicle) then

		--> Variables
		local vehicle = TCBDealer.vehicleTable[currentVehicle:GetNWString("dealerClass")]

		--> Qualify
		if vehicle and vehicle.customCheck and !vehicle.customCheck(ply) then
			TCBDealer.removeVehicle(ply)
			DarkRP.notify(ply, 1, 4, "You no longer qualify for your vehicle.") 
			return
		end

	end
end
hook.Add("OnPlayerChangedTeam", "TCBDealer.playerChanged", TCBDealer.playerChanged)

--[[---------------------------------------------------------
	Dealer Range
-----------------------------------------------------------]]
function TCBDealer.dealerRange(ply, dealer)

	--> Return
	return ply:GetPos():Distance(dealer:GetPos()) <= 200

end